/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockito.internal.progress;


/**
 * Marking interface to be removed when DeprecatedOngoingStubbing gets deleted
 */
public interface IOngoingStubbing {}