package org.mockito.release.notes.versions;

/**
 * Provides the previous version.
 */
public interface PreviousVersion {

    /**
     * The previous version
     */
    String getPreviousVersion();
}
