/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockito.internal.stubbing.defaultanswers;

@SuppressWarnings("unused")
interface HasPrimitiveMethods {
    boolean booleanMethod();
    char charMethod();
    byte byteMethod();
    short shortMethod();
    int intMethod();
    long longMethod();
    float floatMethod();
    double doubleMethod();
}
