/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockitousage.configuration;

/**
 * Some class to mock that is created via Class.forClass
 */
public class ClassToBeMocked { }
