/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockito.invocation;

/**
 * Describes the location of something in the source code.
 */
public interface Location {

    /**
     * @return the location
     */
    String toString();

}
