/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */
package org.mockitoutil;

public interface Assertor<T> {
    
    void assertValue(T value);
}
