package org.mockito.release.notes.improvements;

import org.mockito.release.notes.util.HumanReadable;

/**
 * the improvements
 */
public interface ImprovementSet extends HumanReadable {

    /**
     * human readable representation of the improvements
     */
    String toText();
}
