/*
 * Copyright (c) 2007 Mockito contributors
 * This program is made available under the terms of the MIT License.
 */

package org.mockitousage.examples.use;

public class Article {

    private int numberOfRelatedArticles;

    public int getNumberOfRelatedArticles() {
        return numberOfRelatedArticles;
    }

    public void setNumberOfRelatedArticles(int numberOfRelatedArticles) {
        this.numberOfRelatedArticles = numberOfRelatedArticles;
    }
}
